@@ -85,7 +85,7 @@ function(d, meanfn=c("linear", "constant") ,
          s2.lam=c(0.2,10),		# s2 hierarc inv-gamma prior params (or "fixed")
          tau2.p=c(5,10),	       	# tau2 prior params (initial values) <a0> and <g0>
          tau2.lam=c(0.2,0.1),		# tau2 hierarch inv-gamma prior params (or "fixed")
-         corr=corr,			# correllation model (exp, expsep, matern, sim)
+         corr=corr,			# correlation model (exp, expsep, matern, sim)
          gd=c(0.1, 0.5),                # start vals for nug and d
          nug.p=c(1,1,1,1),		# nug gamma-mix prior params (initial values)
          nug.lam="fixed",		# nug hierarch gamma-mix prior params (or "fixed")
